package com.melot.kkcommon

import android.app.Activity
import android.content.Context
import android.content.Intent
import com.melot.kkbasiclib.KKDefine
import com.melot.kkcommon.util.UrlChecker
import com.melot.kkcommon.util.Util
import com.melot.kkcommon.widget.ActionWebview

/**
 * Created by siwen.hu on 2018/7/3.
 */

class WebViewBuilder {

    var mFrom: Context? = null

    fun from(from: Context): WebViewBuilder {
        mFrom = from
        return this
    }

    private var mUrl: String? = null

    fun url(url: String?): WebViewBuilder {
        mUrl = url
        return this
    }

    private var mTitle: String? = null

    fun title(title: String?): WebViewBuilder {
        mTitle = title
        return this
    }

    private var mShareContent: String? = null

    fun shareContent(title: String?): WebViewBuilder {
        mShareContent = title
        return this
    }

    private var mShowH5Title: Boolean = false

    fun showH5Title(show: Boolean = true): WebViewBuilder {
        mShowH5Title = show
        return this
    }

    fun showH5Title(): WebViewBuilder {
        return showH5Title(true)
    }

    private var mShowRightButton: Boolean = true

    fun hideRightButton(): WebViewBuilder {
        mShowRightButton = false
        return this
    }


    private var mEnterFrom: String? = null

    fun enterFrom(enterFrom: String?): WebViewBuilder {
        mEnterFrom = enterFrom
        return  this
    }

    private var mBuilder: ((i: Intent) -> Unit)? = null

    fun addBuilder(builder: (i: Intent) -> Unit): WebViewBuilder {
        mBuilder = builder
        return this
    }

    fun jump() {
        jumpForResult()
    }

    fun jumpForResult(code: Int = -999) {
        mUrl?.let {
            UrlChecker.check(it, object : UrlChecker.UrlCheckerListener {
                override fun toRoom(finalRoomId: Long, finalRoomSource: Int, finalStreamType: Int) {
                    mFrom?.let {
                        Util.openRoomForceWithEnterFrom(it, finalRoomId, finalRoomId, finalRoomSource, finalStreamType, mEnterFrom, -1, -1)
                    }
                }

                override fun toNameCard(finalRoomId: Long) {
                    mFrom?.let {
                        Util.openNamecard(it,
                                finalRoomId, false, false, "", true)
                    }
                }

                override fun toWeb(url: String) {
                    mFrom?.let {
                        val intent = Intent(it, ActionWebview::class.java)
                        mUrl = url
                        mUrl?.let {
                            intent.putExtra(ActionWebview.WEB_URL, it)
                        }
                        mTitle?.let {
                            intent.putExtra(ActionWebview.WEB_TITLE, it)
                        }
                        mShareContent?.let {
                            intent.putExtra(ActionWebview.WEB_SHARE_CONTNET, it)
                        }
                        mEnterFrom?.let {
                            intent.putExtra(KKDefine.KEY_ROOMFROM, it)
                        }
                        intent.putExtra(ActionWebview.WEB_IS_SHOW_H5_TITLE, mShowH5Title)
                        intent.putExtra(ActionWebview.WEB_NEED_RIGHT_BUTTON, mShowRightButton)
                        mBuilder?.invoke(intent)
                        if (code == -999) {
                            it.startActivity(intent)
                        } else {
                            (it as Activity).let { a ->
                                a.startActivityForResult(intent, code)
                            }
                        }
                    }
                }
            })
        }
    }

}
