package com.melot.kkcommon.util

import android.net.Uri
import com.melot.complib.router.ui.UIRouter
import com.melot.kkbasiclib.KKType
import com.melot.kkbasiclib.callbacks.Callback1
import com.melot.kkcommon.CommonSetting
import com.melot.kkcommon.Global
import com.melot.kkcommon.KKCommonApplication
import com.melot.kkcommon.R
import com.melot.kkcommon.cfg.AppConfig
import com.melot.kkcommon.cfg.ReleaseConfig
import com.melot.kkcommon.sns.httpnew.HttpMessageDump
import com.melot.kkcommon.sns.httpnew.HttpTaskManager
import com.melot.kkcommon.sns.httpnew.reqtask.EncryptAccountReq
import com.melot.kkcommon.sns.httpnew.reqtask.GetUserLiveStateReq
import com.melot.kkcommon.util.Util.getApplicationContext
import com.tencent.mm.opensdk.modelbiz.WXLaunchMiniProgram
import com.tencent.mm.opensdk.openapi.WXAPIFactory
import java.util.regex.Pattern


/**
 * Created by siwen.hu on 2017/8/31.
 */

object UrlChecker {

    private const val M_URL_ROOM_1 = "http://www.kktv1.com/m/?roomid="
    private const val M_URL_ROOM_1s = "https://www.kktv1.com/m/?roomid="
    private const val M_URL_ROOM_5 = "http://www.kktv5.com/m/?roomid="
    private const val M_URL_ROOM_5s = "https://www.kktv5.com/m/?roomid="
    private const val WEB_URL_ROOM_1 = "http://www.kktv1.com/"
    private const val WEB_URL_ROOM_5 = "http://www.kktv5.com/"
    private const val WEB_URL_ROOM_1s = "https://www.kktv1.com/"
    private const val WEB_URL_ROOM_5s = "https://www.kktv5.com/"
    private const val MINI_PROGRAM_PREFIX = "pages/miniactivity"

    private const val KK_ROUTE_PREFIX = "KKComp://"

    private const val KK_JUMP_PREFIX = "KKJump://"

    interface UrlCheckerListener {
        fun toRoom(finalRoomId: Long, finalRoomSource: Int, finalStreamType: Int)

        fun toNameCard(finalRoomId: Long)

        fun toWeb(url: String)
//        fun toMiniProgram(type: String)
    }

    fun check(url: String?, checkerListener: UrlCheckerListener?) {
        if (url.isNullOrBlank()) {
            return
        }
//        var url = "https://mobile.shihanbainian.com/h5/kk_shihan/good_detail.html?id=724"
        url.let {
            if (it.startsWith(KK_JUMP_PREFIX)) {
                openPop(it)
            } else if (it.startsWith(KK_ROUTE_PREFIX)) {
                // UIRouter.getInstance().openUri(it, it, null)
                KKCommonApplication.getInstance().getTopActivity(Callback1 { UIRouter.getInstance().openUri(it, url, null) })
            } else if (it.startsWith(MINI_PROGRAM_PREFIX)) {
                jump2MiniProgram(it)
            } else if (it.contains(AppConfig.getInstance().meshowConfig.specialUrl)) {
                jump2Special(it, checkerListener)
            } else if (it.startsWith(M_URL_ROOM_1) || it.startsWith(M_URL_ROOM_5) || it.startsWith(M_URL_ROOM_1s) || it.startsWith(M_URL_ROOM_5s)) {
                jump2Room(it, checkerListener)
            } else if (it.startsWith(WEB_URL_ROOM_1) || it.startsWith(WEB_URL_ROOM_5) || it.startsWith(WEB_URL_ROOM_1s) || it.startsWith(WEB_URL_ROOM_5s)) {
                jump2Room(it, checkerListener)
            } else {
                checkerListener?.toWeb(it)
            }
        }
    }

    private fun openPop(it: String) {
        if (it.contains("petPop")) {
            HttpMessageDump.getInstance().dispatchInAppTo<Any>(KKType.Where.GoldTask, KKType.AppMessageType.TYPE_OEPN_PET)
        }
    }

    private fun jump2MiniProgram(url: String) {
        val rgex = MINI_PROGRAM_PREFIX + "_(.*?)/"
        val matcher = Pattern.compile(rgex).matcher(url)
        if (matcher.find()) {
            val type = matcher.group(1)
            var newUrl = if (url.contains("?")) "$url&fromApp=1" else "$url?fromApp=1"
            if ("buy" == type) {//java equals
                //wx00e4645d5d846bee
                //关联过appID
                val appId = Global.WEIXIN_APP_ID//Global.WECHAT_PROG_IDS.LIVE_BUY_APPID // 填应用AppId
                val api = WXAPIFactory.createWXAPI(KKCommonApplication.getInstance(), appId)
                val req = WXLaunchMiniProgram.Req()
                //gh_5ff822670ade
                req.userName = Global.WECHAT_PROG_IDS.LIVE_BUY_PROGRAM_ID // 填小程序原始id
                req.path = newUrl  //拉起小程序页面的可带参路径，不填默认拉起小程序首页
                if (ReleaseConfig.DEBUG) {// 可选打开 开发版，体验版和正式版
                    req.miniprogramType = WXLaunchMiniProgram.Req.MINIPROGRAM_TYPE_PREVIEW
                } else {
                    req.miniprogramType = WXLaunchMiniProgram.Req.MINIPTOGRAM_TYPE_RELEASE
                }
                api.sendReq(req)
            }
        }
    }

    private fun jump2Special(url: String?, checkerListener: UrlCheckerListener?) {
        Util.showToast(getApplicationContext().getString(R.string.kk_go_to_third_party_page))
        CommonSetting.getInstance().shiHanUrl = url
        if (!CommonSetting.getInstance().isVisitor) {
            HttpTaskManager.getInstance().request(EncryptAccountReq(CommonSetting.getInstance().userId, CommonSetting.getInstance().token, EncryptAccountReq.TYPE_SHIHAN) {
                if (it.isSuccess) {
                    it.value?.let {
                        checkerListener?.toWeb(Util.getShiHanUrl(it.userId, it.token))
                    }
                }
            })
        } else {
            checkerListener?.toWeb(Util.getShiHanUrl("", ""))
        }
    }

    private fun jump2Room(url: String?, checkerListener: UrlCheckerListener?) {
        val uri = Uri.parse(url)
        try {
            val roomIdStr = uri.getQueryParameter("roomid")
            var roomId: Long = 0
            if (roomIdStr.isNullOrBlank().not())
                roomId = roomIdStr.toLong()
            val roomsourceStr = uri.getQueryParameter("roomSource")
            var roomSource = KKType.RoomSourceType.TYPE_MESHOW
            if (roomsourceStr.isNullOrBlank().not()) {
                roomSource = roomsourceStr.toInt()
            }
            val screentypeStr = uri.getQueryParameter("screenType")
            var streamType = KKType.StreamType.TYPE_HORI
            if (screentypeStr.isNullOrBlank().not()) {
                streamType = screentypeStr.toInt()
            }
            if (roomId > 0) {
                val finalStreamType = streamType
                val finalRoomSource = roomSource
                val finalRoomId = roomId
                HttpTaskManager.getInstance().request(GetUserLiveStateReq(roomId) { pp ->
                    if (pp.liveType > 0) {
                        checkerListener?.toRoom(finalRoomId, finalRoomSource, finalStreamType)
                    } else {
                        checkerListener?.toNameCard(finalRoomId)
                    }
                })
            } else {
                url?.let {
                    checkerListener?.toWeb(it)
                }
            }
        } catch (e: NumberFormatException) {
            e.printStackTrace()
        }
    }

}
