package com.melot.meshow.room.chat.util

import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.drawable.BitmapDrawable
import android.os.Handler
import android.os.Looper
import android.support.annotation.ColorInt
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.view.Gravity
import android.view.View
import com.bumptech.glide.Glide
import com.bumptech.glide.request.animation.GlideAnimation
import com.bumptech.glide.request.target.SimpleTarget
import com.melot.kkcommon.Global
import com.melot.kkcommon.KKCommonApplication
import com.melot.kkbasiclib.KKType.MemberIdentity.*
import com.melot.kkbasiclib.callbacks.Callback1
import com.melot.kkcommon.OpenPlatformMagic
import com.melot.kkcommon.room.chat.CenterImageSpan
import com.melot.kkcommon.room.flyway.NameSpan
import com.melot.kkcommon.room.gift.GiftDataManager
import com.melot.kkcommon.shop.Vip
import com.melot.kkcommon.util.ResourceUtil
import com.melot.kkcommon.util.Util
import com.melot.meshow.room.R
import com.melot.meshow.room.util.MeshowUtil

/**
 * Created by siwen.hu on 2018/6/19.
 */

class MessageBuilder {

    companion object {
        val MSG_NORMAL_USER_NAME = Color.parseColor("#D3870B")
    }


    var ssbs: MutableList<MessageItem>? = mutableListOf()
    var isRetry: Boolean = false

    var resultCallback: ((SpannableStringBuilder) -> Unit)? = null


    /**添加神秘人icon**/

    @Synchronized
    fun addMysIcon(): MessageBuilder {
        val i = MessageItem(SpannableStringBuilder())
        i.item.append("isMys ")
        val vipBmps = ResourceUtil.getDrawable(R.drawable.kk_room_stealth_v_icon) as BitmapDrawable
        vipBmps.gravity = Gravity.CENTER
        vipBmps.setBounds(0, 0, (14 * Global.density).toInt(), (14 * Global.density).toInt())
        i.item.setSpan(CenterImageSpan(vipBmps), 0, 5, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        ssbs?.add(i)
        return this
    }

    /**添加主播等级icon**/

    @Synchronized
    fun addActorLvIcon(actorLevel: Int): MessageBuilder {
        if (actorLevel > 0) {
            val rLvId = ResourceUtil.getActorRes(actorLevel)
            val rlBmps = ResourceUtil.getDrawable(rLvId) as BitmapDrawable
            rlBmps.gravity = Gravity.FILL
            var height = Util.dip2px(14f)
            var width = height * rlBmps.bitmap.width / rlBmps.bitmap.height
            rlBmps.setBounds(0, 0, width, height)

            val i = MessageItem(SpannableStringBuilder())
            i.item.append("actorLv ")
            i.item.setSpan(CenterImageSpan(rlBmps), 0, 7, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            //财富等级 end
            ssbs?.add(i)
        }
        return this
    }

    @Synchronized
    fun addIntimacyLvIcon(intimacyLv: Int): MessageBuilder {
        if (intimacyLv > 0) {
            val resourceLv = ResourceUtil.getIntimacyLevel(intimacyLv)
            val rlBmps = ResourceUtil.getDrawable(resourceLv) as BitmapDrawable
            rlBmps.gravity = Gravity.FILL
            var height = Util.dip2px(14f)
            var width = height * rlBmps.bitmap.width / rlBmps.bitmap.height
            rlBmps.setBounds(0, 0, width, height)

            val i = MessageItem(SpannableStringBuilder())
            i.item.append("intimacyLv ")
            i.item.setSpan(CenterImageSpan(rlBmps), 0, "intimacyLv".length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            ssbs?.add(i)
        }
        return this
    }

    fun onBuild(onBuild: ((SpannableStringBuilder) -> Unit)?): MessageBuilder {
        this.resultCallback = onBuild
        return this
    }

    /**添加财富等级icon**/

    @Synchronized
    fun addRichLvIcon(richLevel: Int): MessageBuilder {
        return addRichLvIcon(richLevel, 0)
    }

    /**
     * 会检测是否有自定
     */
    @Synchronized
    fun addRichLvIcon(richLevel: Int, userId: Long): MessageBuilder {
        if (richLevel >= 0) {
            val i = MessageItem(SpannableStringBuilder())
            ssbs?.add(i)
            ResourceUtil.loadRichIcon(richLevel, userId) {
                if (it == null) {
                    val rLvId = ResourceUtil.getRichRes(richLevel)
                    val rLvWidth = MeshowUtil.getNameCardRichWidth(richLevel)
                    val rlBmps = ResourceUtil.getDrawable(rLvId) as BitmapDrawable
                    rlBmps.gravity = Gravity.CENTER
                    rlBmps.setBounds(0, 0, rLvWidth, (14 * Global.density).toInt())
                    i.item.append("richLv ")
                    i.item.setSpan(CenterImageSpan(rlBmps), 0, 6, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                } else {
                    val bd = BitmapDrawable(KKCommonApplication.getInstance().resources, it)
                    bd.gravity = Gravity.FILL
                    var height = Util.dip2px(14f)
                    var width = height * it.width / it.height
                    bd.setBounds(0, 0, width, height)
                    i.item.append("rich ").setSpan(CenterImageSpan(bd), 0, "rich".length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                    reBuild()
                }
            }
        }
        return this
    }

    @Synchronized
    fun addStarIcon(starLevel: Int): MessageBuilder {
        if (starLevel > 0) {
            val newSsb = SpannableStringBuilder()
            val bd = ResourceUtil.getDrawable("kk_star_level_$starLevel") as BitmapDrawable
            bd.gravity = Gravity.CENTER
            bd.setBounds(0, 0, (14f * Global.density).toInt(), (14 * Global.density).toInt())
            newSsb.append("startlv ").setSpan(CenterImageSpan(bd), 0, "startlv".length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            val i = MessageItem(newSsb)
            ssbs?.add(i)
        }
        return this
    }

    /**添加贵族勋章icon**/

    @Synchronized
    fun addNobilityIcon(nobilityLv: Int): MessageBuilder {
        if (nobilityLv < 1) return this
        OpenPlatformMagic.getNobalInfo(nobilityLv, {
            val item = MessageItem(SpannableStringBuilder(), false)
            downLoadNobal(it, item)
            ssbs?.add(item)
        }, {
            val newSsb = SpannableStringBuilder()
            val bd = it;
            bd.gravity = Gravity.CENTER
            bd.setBounds(0, 0, (3f * 14f * Global.density).toInt(), (14 * Global.density).toInt())
            newSsb.append("reCharge ").setSpan(CenterImageSpan(bd), 0, "reCharge".length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            var item = MessageItem(newSsb)
            ssbs?.add(item)
        });
        return this
    }

    private fun downLoadNobal(nobalUrl: String, item: MessageItem) {
        var mark = 1
        val doDown: () -> Unit = {
            Glide.with(KKCommonApplication.getInstance()).load(nobalUrl).asBitmap().into(object : SimpleTarget<Bitmap>() {
                override fun onResourceReady(bmp: Bitmap?, glideAnimation: GlideAnimation<in Bitmap>) {
                    if (bmp == null) {
                        return
                    }
                    val height = Util.dip2px(17f)
                    val width = height * bmp.width / bmp.height
                    if (bmp.isRecycled) {
                        return
                    }
                    val newBmp = Bitmap.createScaledBitmap(bmp, width, height, true)
                    val bd = BitmapDrawable(KKCommonApplication.getInstance().resources, newBmp)
                    bd.gravity = Gravity.CENTER
                    bd.setBounds(0, 0, width, height)
                    item.item.append("nobal ").setSpan(CenterImageSpan(bd), 0, "nobal".length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                    item.ok = true
                    if (mark == 2) {
                        //异步下载图片完成
                        reBuild()
                    }
                }
            })
        }
        if (!Util.isOnMainThread()) {
            Handler(Looper.getMainLooper()).post(doDown)
        } else {
            doDown.invoke()
        }
        mark = 2
    }

    /**添加用户身份icon**/

    @Synchronized
    fun addIdentityIcon(identity: Int, vipId: Int): MessageBuilder {
        val identityResId = when (identity) {
            /**管理员 */
            ROOM_IDENTITY_ADMIN -> when (vipId) {
                Vip.NOMAL_VIP_ID -> R.drawable.kk_room_admin2_icon
                Vip.SUPER_VIP_ID -> R.drawable.kk_room_admin3_icon
                else -> R.drawable.kk_room_admin1_icon
            }
            /**房主 */
            ROOM_IDENTITY_OWNER -> R.drawable.kk_room_owner_icon
            /**官方 */
            ROOM_IDENTITY_OFFICIAL -> R.drawable.kk_room_offical_icon
            /**代理 */
            ROOM_IDENTITY_AGENCY -> R.drawable.kk_room_agency_icon
            /**巡官 */
            ROOM_IDENTITY_INSPECTOR -> R.drawable.kk_room_inspector_icon
            /**技 */
            ROOM_IDENTITY_TRAINING -> R.drawable.kk_room_training_icon
            /**运营 */
            ROOM_IDENTITY_OPERATING -> R.drawable.kk_room_operating_icon
            /**守护 */
            ROOM_IDENTITY_GUARD -> R.drawable.kk_room_guard_icon
            else -> 0
        }
        if (identityResId > 0) {
            val i = MessageItem(SpannableStringBuilder())
            i.item.append("ident ")
            val vipBmps = ResourceUtil.getDrawable(identityResId) as BitmapDrawable
            vipBmps.gravity = Gravity.CENTER
            vipBmps.setBounds(0, 0, (14 * Global.density).toInt(), (14 * Global.density).toInt())
            i.item.setSpan(CenterImageSpan(vipBmps), 0, i.item.length - 1, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            ssbs?.add(i)
        }
        return this
    }

    /**添加字符串**/

    @Synchronized
    fun addString(s: CharSequence, @ColorInt color: Int? = Color.WHITE): MessageBuilder {
        val temp = SpannableStringBuilder()
        temp.append(s)
        temp.setSpan(ForegroundColorSpan(color ?: Color.WHITE),
                0, s.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        val item = MessageItem(temp)
        ssbs?.add(item)
        return this
    }


    @Synchronized
    fun addString(s: CharSequence): MessageBuilder {
        return addString(s, null)
    }

    @Synchronized
    fun addSpannableStringBuilder(s: SpannableStringBuilder): MessageBuilder {
        val item = MessageItem(s)
        ssbs?.add(item)
        return this
    }

    /**添加用户名字**/

    @Synchronized
    fun addName(userId: Long, name: CharSequence, @ColorInt color: Int? = MSG_NORMAL_USER_NAME, listener: ((Long) -> Unit)?): MessageBuilder {
        val fromNameSpan = object : NameSpan() {
            override fun onClick(widget: View) {
                super.onClick(widget)
                listener?.invoke(userId)
            }
        }
        fromNameSpan.nameColor = color ?: MSG_NORMAL_USER_NAME
        val temp = SpannableStringBuilder()
        temp.append(name)
        temp.setSpan(fromNameSpan, 0, temp.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        val item = MessageItem(temp)
        ssbs?.add(item)
        return this
    }

    /**添加礼物icon**/

    @Synchronized
    fun addGiftIcon(giftId: Int): MessageBuilder {
        val item = MessageItem(SpannableStringBuilder(), false)
        downLoadGift(giftId, item)
        ssbs?.add(item)
        return this
    }


    @Synchronized
    private fun reBuild(): SpannableStringBuilder {
        val result = SpannableStringBuilder()
        ssbs?.forEach {
            if (it.ok) result.append(it.item)
        }
        resultCallback?.invoke(result)
        return result
    }

    /**
     * 没有异常请求 可以直接调用build同步获取结果
     */
    @Synchronized
    fun build(): SpannableStringBuilder {
        val result = SpannableStringBuilder()
        ssbs?.forEach {
            if (it.ok) result.append(it.item)
        }
        return result
    }


    private fun downLoadGift(giftId: Int, item: MessageItem, skipMemory: Boolean = false) {
        var mark = 1
        val doDown: () -> Unit = {
            Glide.with(KKCommonApplication.getInstance()).load(GiftDataManager.getInstance().getThumb35_35(giftId)).asBitmap().skipMemoryCache(skipMemory).into(object : SimpleTarget<Bitmap>() {
                override fun onResourceReady(bmp: Bitmap?, glideAnimation: GlideAnimation<in Bitmap>) {
                    if (bmp == null) {
                        return
                    }
                    val height = Util.dip2px(17f)
                    val width = height * bmp.width / bmp.height
                    if (bmp.isRecycled) {
                        if (!isRetry) {
                            isRetry = true
                            downLoadGift(giftId, item, true)
                        }
                        return
                    }
                    val newBmp = Bitmap.createScaledBitmap(bmp, width, height, true)
                    val bd = BitmapDrawable(KKCommonApplication.getInstance().resources, newBmp)
                    bd.gravity = Gravity.CENTER
                    bd.setBounds(0, 0, width, height)
                    item.item.append("gift").setSpan(CenterImageSpan(bd), 0, "gift".length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                    item.ok = true
                    if (mark == 2) {
                        //异步下载图片完成
                        reBuild()
                    }
                }
            })
        }
        if (!Util.isOnMainThread()) {
            Handler(Looper.getMainLooper()).post(doDown)
        } else {
            doDown.invoke()
        }
        mark = 2
    }

}

data class MessageItem(var item: SpannableStringBuilder, var ok: Boolean = true)
